//+------------------------------------------------------------------+
//|                   17 Nov 2011                  Nonlag Schaff.mq4 |
//|                                                           mladen |
//+------------------------------------------------------------------+
#property copyright "mladen"
#property link      "mladenfx@gmail.com"

#property indicator_separate_window
#property indicator_buffers    3
#property indicator_color1     Lime//ForestGreen
#property indicator_color2     Red//Crimson
#property indicator_color3     Yellow//Crimson
#property indicator_width1     2
#property indicator_width2     2
#property indicator_width3     2
#property indicator_level1     15//25
#property indicator_level2     85//75

#property indicator_levelcolor SlateGray
#property indicator_levelstyle STYLE_DOT
#property indicator_levelwidth 1
//
//
//
//
//

extern string TimeFrame        = "current time frame";
extern int    FastLength       = 23;//15 (Prof Twomey's settings @ 23(I used 28))
extern int    SlowLength       = 50;//34
extern int    StcPeriod        = 10;//5
extern int    MacdPrice        = PRICE_CLOSE;
extern bool   Interpolate      = true;
extern bool   MultiColor       = true;

extern string note             = "turn on Alert = true; turn off = false";
extern bool   alertsOn         = true;
extern bool   alertsOnCurrent  = true;
extern bool   alertsMessage    = true;
extern bool   alertsSound      = true;
extern bool   alertsEmail      = false;
extern string soundfile        = "alert2.wav";



//
//
//
//
//

double stcBuffer[];
double stcBufferUA[];
double stcBufferUB[];
double macdBuffer[];
double fastKBuffer[];
double fastDBuffer[];
double fastKKBuffer[];
double trend[];
double alphas[];

//
//
//
//
//

string indicatorFileName;
bool   calculateSchaff;
bool   returnBars;
int    timeFrame;

//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
//
//
//
//
//

int init()
{
   
   IndicatorBuffers(8);
   SetIndexBuffer(0,stcBuffer);
   SetIndexBuffer(1,stcBufferUA);
   SetIndexBuffer(2,stcBufferUB);
   SetIndexBuffer(3,macdBuffer);
   SetIndexBuffer(4,fastKBuffer);
   SetIndexBuffer(5,fastDBuffer);
   SetIndexBuffer(6,fastKKBuffer);
   SetIndexBuffer(7,trend);
   //
   //
   //
   //
   //
   
   indicatorFileName = WindowExpertName();
   calculateSchaff   = (TimeFrame=="calculateSchaff"); if (calculateSchaff) return(0);
   returnBars        = (TimeFrame=="returnBars");     if (returnBars)     return(0);
   timeFrame         = stringToTimeFrame(TimeFrame);
    
   //
   //
   //
   //
   //
    
   IndicatorShortName(timeFrameToString(timeFrame)+" Non lag Schaff Trend Cycle  ("+FastLength+","+StcPeriod+","+SlowLength+")");
  
   return (0);
}
      

//
//
//
//
//

int deinit() { return(0); }

//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
//
//
//
//
//

int start()
{
   int counted_bars=IndicatorCounted();
   int i,n,k,limit;

   if(counted_bars<0) return(-1);
   if(counted_bars>0) counted_bars--;
         limit = MathMin(Bars-counted_bars,Bars-1);
         if (returnBars) { stcBuffer[0] = MathMin(limit+1,Bars-1); return(0); }        
   //
   //
   //
   //
   //
   
   if (calculateSchaff || timeFrame == Period())
   {
   if (MultiColor && trend[limit] == 1) CleanPoint(limit,stcBufferUA,stcBufferUB);
   
   //
   //
   //
   //
   //        
  
      for(i = limit; i >= 0; i--)
      {
          stcBufferUA[i] = EMPTY_VALUE;
          stcBufferUB[i] = EMPTY_VALUE;
          macdBuffer[i] = iNoLagMa(iMA(NULL,0,1,0,MODE_SMA,MacdPrice,i),FastLength,alphas,0,i)-
                          iNoLagMa(iMA(NULL,0,1,0,MODE_SMA,MacdPrice,i),SlowLength,alphas,1,i);

         //
         //
         //
         //
         //
         
         double lowMacd  = minValue(macdBuffer,i);
         double highMacd = maxValue(macdBuffer,i)-lowMacd;
            if (highMacd > 0)
                  fastKBuffer[i] = 100*((macdBuffer[i]-lowMacd)/highMacd);
            else  fastKBuffer[i] = fastKBuffer[i+1];
                  fastDBuffer[i] = fastDBuffer[i+1]+0.5*(fastKBuffer[i]-fastDBuffer[i+1]);
               
         //
         //
         //
         //
         //
                     
         double lowStoch  = minValue(fastDBuffer,i);
         double highStoch = maxValue(fastDBuffer,i)-lowStoch;
            if (highStoch > 0)
                  fastKKBuffer[i] = 100*((fastDBuffer[i]-lowStoch)/highStoch);
            else  fastKKBuffer[i] = fastKKBuffer[i+1];
                  stcBuffer[i]    = stcBuffer[i+1]+0.5*(fastKKBuffer[i]-stcBuffer[i+1]);
                  trend[i]        = trend[i+1];
         if (stcBuffer[i] > stcBuffer[i+1]) trend[i] =  1;
         if (stcBuffer[i] < stcBuffer[i+1]) trend[i] = -1;
         if (MultiColor && !calculateSchaff && trend[i]==-1) PlotPoint(i,stcBufferUA,stcBufferUB,stcBuffer);
       }         
      manageAlerts();  
      return(0);
   }
   
   //
   //
   //
   //
   //
   
   limit = MathMax(limit,MathMin(Bars,iCustom(NULL,timeFrame,indicatorFileName,"returnBars",0,0)*timeFrame/Period()));
   if (MultiColor && trend[limit]==-1) CleanPoint(limit,stcBufferUA,stcBufferUB);      
   for (i=limit;i>=0; i--)
   {
      int y = iBarShift(NULL,timeFrame,Time[i]);

         stcBufferUA[i] = EMPTY_VALUE;
         stcBufferUB[i] = EMPTY_VALUE;
         stcBuffer[i]   = iCustom(NULL,timeFrame,indicatorFileName,"calculateSchaff",FastLength,SlowLength,StcPeriod,MacdPrice,0,y);
         trend[i]       = iCustom(NULL,timeFrame,indicatorFileName,"calculateSchaff",FastLength,SlowLength,StcPeriod,MacdPrice,7,y);
         
         //
         //
         //
         //
         //
         
         if (!Interpolate || y==iBarShift(NULL,timeFrame,Time[i-1])) continue;

         //
         //
         //
         //
         //

         datetime time = iTime(NULL,timeFrame,y);
            for(n = 1; i+n < Bars && Time[i+n] >= time; n++) continue;	
            for(k = 1; k < n; k++)
              stcBuffer[i+k] = stcBuffer[i] + (stcBuffer[i+n] - stcBuffer[i]) * k/n;
   }
   if (MultiColor) for (i=limit;i>=0;i--) if (trend[i]==-1) PlotPoint(i,stcBufferUA,stcBufferUB,stcBuffer);

   //
   //
   //
   //
   //
   manageAlerts(); 
   return(0);
}

//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
//
//
//
//
//

double values[][3];
double prices[][1];

#define Pi       3.14159265358979323846264338327950288
#define _length  0
#define _len     1
#define _weight  2

//
//
//
//
//

double iNoLagMa(double price, int length, double &alphasArray[], int forValue, int i)
{
   if (ArrayRange(prices,0) != Bars) ArrayResize(prices,Bars);
            int r = Bars-i-1;  prices[r][forValue]=price;
   if (length<3 || r<3) return(prices[r][forValue]);
   
   //
   //
   //
   //
   //
   
   if (ArrayRange(values,0)<(forValue+1) || values[forValue][_length] != length)
   {
      double Cycle = 4.0;
      double Coeff = 3.0*Pi;
      int    Phase = length-1;
      
      if (ArrayRange(values,0)<forValue+1) ArrayResize(values,forValue+1);

         values[forValue][_length] = length;
         values[forValue][_len]    = length*4 + Phase;  
         values[forValue][_weight] = 0;  
         ArrayResize(alphasArray,values[forValue][_len]);

         for (int k=0; k<values[forValue][_len]-1; k++)
         {
            if (k<=Phase-1) 
                 double t = 1.0 * k/(Phase-1);
            else        t = 1.0 + (k-Phase+1)*(2.0*Cycle-1.0)/(Cycle*length-1.0); 
            double beta = MathCos(Pi*t);
            double g = 1.0/(Coeff*t+1); if (t <= 0.5 ) g = 1;
      
            alphasArray[k]             = g * beta;
            values[forValue][_weight] += alphasArray[k];
         }
   }
   
   //
   //
   //
   //
   //
   
   if (values[forValue][_weight]>0)
   {
      int    len = values[forValue][_len];
      double sum = 0;
           for (k=0; k < len-1; k++) sum += alphasArray[k]*prices[r-k][forValue];      
           return( sum / values[forValue][_weight]);
   }
   else return(0);           
}

//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
//
//
//
//
//

void manageAlerts()
{
   if (!calculateSchaff && alertsOn)
   {
      if (alertsOnCurrent)
           int whichBar = 0;
      else     whichBar = 1; whichBar = iBarShift(NULL,0,iTime(NULL,timeFrame,whichBar));
      if (trend[whichBar] != trend[whichBar+1])
      {
         if (trend[whichBar] ==  1) doAlert(whichBar,"up");
         if (trend[whichBar] == -1) doAlert(whichBar,"down");
      }
   }
}

//
//
//
//
//

void doAlert(int forBar, string doWhat)
{
   static string   previousAlert="nothing";
   static datetime previousTime;
   string message;
   
   if (previousAlert != doWhat || previousTime != Time[forBar]) {
       previousAlert  = doWhat;
       previousTime   = Time[forBar];

       //
       //
       //
       //
       //

       message =  StringConcatenate(Symbol()," ",timeFrameToString(timeFrame)," at ",TimeToStr(TimeLocal(),TIME_SECONDS)," Non Lag Schaff Trend Cycle changed direction to ",doWhat);
          if (alertsMessage) Alert(message);
          if (alertsEmail)   SendMail(StringConcatenate(Symbol()," Non Lag Schaff Trend Cycle "),message);
          if (alertsSound)   PlaySound("alert2.wav");
   }
}

//+-------------------------------------------------------------------
//|                                                                  
//+-------------------------------------------------------------------
//
//
//
//
//

void CleanPoint(int i,double& first[],double& second[])
{
   if ((second[i]  != EMPTY_VALUE) && (second[i+1] != EMPTY_VALUE))
        second[i+1] = EMPTY_VALUE;
   else
      if ((first[i] != EMPTY_VALUE) && (first[i+1] != EMPTY_VALUE) && (first[i+2] == EMPTY_VALUE))
          first[i+1] = EMPTY_VALUE;
}

//
//
//
//
//

void PlotPoint(int i,double& first[],double& second[],double& from[])
{
   if (first[i+1] == EMPTY_VALUE)
      {
         if (first[i+2] == EMPTY_VALUE) {
                first[i]   = from[i];
                first[i+1] = from[i+1];
                second[i]  = EMPTY_VALUE;
            }
         else {
                second[i]   =  from[i];
                second[i+1] =  from[i+1];
                first[i]    = EMPTY_VALUE;
            }
      }
   else
      {
         first[i]  = from[i];
         second[i] = EMPTY_VALUE;
      }
}
  

string sTfTable[] = {"M1","M5","M15","M30","H1","H4","D1","W1","MN"};
int    iTfTable[] = {1,5,15,30,60,240,1440,10080,43200};

//
//
//
//
//

int stringToTimeFrame(string tfs)
{
   tfs = stringUpperCase(tfs);
   for (int i=ArraySize(iTfTable)-1; i>=0; i--)
         if (tfs==sTfTable[i] || tfs==""+iTfTable[i]) return(MathMax(iTfTable[i],Period()));
                                                      return(Period());
}
string timeFrameToString(int tf)
{
   for (int i=ArraySize(iTfTable)-1; i>=0; i--) 
         if (tf==iTfTable[i]) return(sTfTable[i]);
                              return("");
}

//
//
//
//
//

string stringUpperCase(string str)
{
   string   s = str;

   for (int length=StringLen(str)-1; length>=0; length--)
   {
      int char = StringGetChar(s, length);
         if((char > 96 && char < 123) || (char > 223 && char < 256))
                     s = StringSetChar(s, length, char - 32);
         else if(char > -33 && char < 0)
                     s = StringSetChar(s, length, char + 224);
   }
   return(s);
}
//
//
//
//
//

double minValue(double& array[],int shift)
{
   double minValue = array[shift];
            for (int i=1; i<StcPeriod; i++) minValue = MathMin(minValue,array[shift+i]);
   return(minValue);
}
double maxValue(double& array[],int shift)
{
   double maxValue = array[shift];
            for (int i=1; i<StcPeriod; i++) maxValue = MathMax(maxValue,array[shift+i]);
   return(maxValue);
}


